@@ -31,9 +31,7 @@ __all__ = [
 ]
 
 
-@deprecated(
-    "0.6.1", alternative="interactive_plot with the ax argument", name="heck", removal="0.7.0"
-)
+@deprecated("0.6.1", alternative="interactive_plot with the ax argument", name="", removal="0.8.0")
 def interactive_plot_factory(
     ax,
     f,
@@ -789,7 +787,7 @@ def interactive_imshow(
     use_ipywidgets = ipympl or force_ipywidgets
     slider_format_strings = create_slider_format_dict(slider_format_string, use_ipywidgets)
 
-    def update(change, label, key):
+    def update(change, key, label):
         if label:
             # continuous
             params[key] = kwargs[key][change["new"]]
